# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"is_3dlut_row": "augment_PIL-img_filters.ipynb",
         "read_lut": "augment_PIL-img_filters.ipynb",
         "ApplyPILFilter": "augment_PIL-img_filters.ipynb",
         "albu_augment": "augment_albumentations.ipynb",
         "AlbumentationsWrapper": "augment_albumentations.ipynb",
         "BlurringTfms": "augment_albumentations.ipynb",
         "StyleTfms": "augment_albumentations.ipynb",
         "WeatherTfms": "augment_albumentations.ipynb",
         "NoiseTfms": "augment_albumentations.ipynb",
         "ColorTonesTfms": "augment_albumentations.ipynb",
         "ColorChannelTfms": "augment_albumentations.ipynb",
         "LightingTfms": "augment_albumentations.ipynb",
         "OtherTfms": "augment_albumentations.ipynb",
         "Tfms": "augment_albumentations.ipynb",
         "KorniaBase": "augment_kornia.ipynb",
         "MotionBlur": "augment_kornia.ipynb",
         "ColorJitter": "augment_kornia.ipynb",
         "RandomRotation": "augment_kornia.ipynb",
         "MedianBlur": "augment_kornia.ipynb",
         "HFlip": "augment_kornia.ipynb",
         "VFlip": "augment_kornia.ipynb",
         "RandomGrayscale": "augment_kornia.ipynb",
         "RandomPerspective": "augment_kornia.ipynb",
         "torch_to_onnx": "export_onnx.ipynb",
         "ClassificationInterpretationEx": "interpret_classification-interpretation.ipynb",
         "ClassificationInterpretationEx.plot_confusion_matrix": "interpret_classification-interpretation.ipynb",
         "ClassificationInterpretationEx.plot_accuracy": "interpret_classification-interpretation.ipynb",
         "ClassificationInterpretationEx.plot_label_confidence": "interpret_classification-interpretation.ipynb",
         "ClassificationInterpretationEx.plot_top_losses_grid": "interpret_classification-interpretation.ipynb",
         "ClassificationInterpretationEx.plot_lowest_losses_grid": "interpret_classification-interpretation.ipynb",
         "ClassificationInterpretationEx.print_classification_report": "interpret_classification-interpretation.ipynb",
         "intersection": "interpret_compare-models.ipynb",
         "compare_venn": "interpret_compare-models.ipynb",
         "Hook": "interpret_gradcam.ipynb",
         "HookBwd": "interpret_gradcam.ipynb",
         "to_cuda": "interpret_gradcam.ipynb",
         "get_label_idx": "interpret_gradcam.ipynb",
         "get_target_layer": "interpret_gradcam.ipynb",
         "compute_gcam_items": "interpret_gradcam.ipynb",
         "compute_gcam_map": "interpret_gradcam.ipynb",
         "plt2pil": "interpret_gradcam.ipynb",
         "plt_decoded": "interpret_gradcam.ipynb",
         "plot_gcam": "interpret_gradcam.ipynb",
         "Learner.gradcam": "interpret_gradcam.ipynb",
         "FocalLoss": "loss_functions.ipynb",
         "FocalLossFlat": "loss_functions.ipynb",
         "TfmdDL.set_font_path": "show_data.ipynb",
         "draw_label": "show_data.ipynb",
         "TfmdDL.show_batch_grid": "show_data.ipynb",
         "open_image": "utils.ipynb",
         "img2arraylike": "utils.ipynb",
         "make_img_grid": "utils.ipynb",
         "PIL.Image.Image.draw_labels": "utils.ipynb",
         "color": "utils.ipynb"}

modules = ["augment/pil_filters.py",
           "augment/albumentations.py",
           "augment/kornia.py",
           "export/onnx.py",
           "interpret/interpret.py",
           "interpret/compare.py",
           "interpret/gradcam.py",
           "loss_funcs.py",
           "show_data.py",
           "utils.py"]

doc_url = "https://Synopsis.github.io/fastai_amalgam/"

git_url = "https://github.com/Synopsis/fastai_amalgam/tree/master/"

def custom_doc_links(name): return None
